@@ -2,10 +2,13 @@
 #
 #  __init__.py
 """
-Enhanced Sphinx TocTree that shows classes and functions as if they were sections.
+Enhanced Sphinx TocTree which shows classes and functions as if they were sections.
 """
 #
-#  Copyright © {{ copyright_years }} {{ author }} <{{ email }}>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#
+#  Based on Sphinx
+#  Copyright (c) 2007-2020 by the Sphinx team.
 #
 #  Redistribution and use in source and binary forms, with or without modification,
 #  are permitted provided that the following conditions are met:
@@ -32,7 +35,7 @@ Enhanced Sphinx TocTree that shows classes and functions as if they were section
 # stdlib
 import os
 from pprint import pprint
-from typing import Any, Dict, List, Optional, Set, Type, TypeVar
+from typing import Any, Dict, Iterable, List, Optional, Type, TypeVar
 
 # 3rd party
 from docutils import nodes
@@ -41,18 +44,17 @@ from sphinx.application import Sphinx
 from sphinx.environment.adapters.toctree import TocTree
 from sphinx.environment.collectors.toctree import TocTreeCollector
 from sphinx.transforms import SphinxContentsFilter
-from sphinx.util import logging
+from sphinx.util import logging, texescape
+from sphinx.writers.latex import LaTeXTranslator
 
 __author__: str = "Dominic Davis-Foster"
-__copyright__: str = "2020 Dominic Davis-Foster"
+__copyright__: str = "2020-2021 Dominic Davis-Foster"
 __license__: str = "BSD"
-__version__: str = "0.1.0"
+__version__: str = "0.5.0"
 __email__: str = "dominic@davis-foster.co.uk"
 
 __all__ = ["TocTreePlusCollector", "setup"]
 
-# For type hinting install docutils-stubs
-
 N = TypeVar('N')
 
 logger = logging.getLogger(__name__)
@@ -73,14 +75,14 @@ class TocTreePlusCollector(TocTreeCollector):
 		"""
 		Build a TOC from the doctree and store it in the inventory.
 
-		:param app:
+		:param app: The Sphinx application.
 		:param doctree:
 		"""
 
 		docname = app.env.docname
 		numentries = [0]  # nonlocal again...
 
-		def traverse_in_section(node: nodes.Element, cls: "Type[N]") -> List[N]:
+		def traverse_in_section(node: nodes.Element, cls: Type[N]) -> List[N]:
 			"""
 			Like traverse(), but stay within the same section.
 
@@ -113,6 +115,7 @@ class TocTreePlusCollector(TocTreeCollector):
 
 			entries: List[nodes.Element] = []
 			item: nodes.Element
+			toctree_plus_types = set(app.env.config.toctree_plus_types)
 
 			for sectionnode in node:
 				# find all toctree nodes in this section and add them
@@ -132,7 +135,7 @@ class TocTreePlusCollector(TocTreeCollector):
 						# as it is the file's title anyway
 						anchorname = ''
 					else:
-						anchorname = '#' + sectionnode["ids"][0]
+						anchorname = f'#{sectionnode["ids"][0]}'
 
 					numentries[0] += 1
 
@@ -160,14 +163,20 @@ class TocTreePlusCollector(TocTreeCollector):
 					# Add class, function and method directives to toctree.
 					# (doesn't currently work for method directives - are they nested?)
 
-					if sectionnode.attributes["objtype"] in set(app.env.config.toctree_plus_types):
+					if sectionnode.attributes["objtype"] in toctree_plus_types:
+
 						attributes = sectionnode.children[0].attributes
 						if not attributes["ids"]:
 							# Has no anchor
 							continue
 
 						title = attributes.get("fullname", sectionnode.children[0].astext())
-						anchorname = '#' + attributes["ids"][0]
+
+						if sectionnode.attributes["objtype"] in {"method", "attribute"}:
+							# TODO: remove special case
+							title = title.split('.', 1)[-1]
+
+						anchorname = f'#{attributes["ids"][0]}'
 
 						reference = nodes.reference(
 								'',
@@ -180,7 +189,13 @@ class TocTreePlusCollector(TocTreeCollector):
 						para = addnodes.compact_paragraph('', '', reference)
 						item = nodes.list_item('', para)
 
+						sub_item = build_toc(sectionnode.children[1], depth + 1)
+
+						if sub_item:
+							item += sub_item
+
 						entries.append(item)
+
 					elif TOCTREE_PLUS_DEBUG:
 						print(sectionnode)
 						pprint(sectionnode.attributes["objtype"])
@@ -193,6 +208,7 @@ class TocTreePlusCollector(TocTreeCollector):
 						entries.append(onlynode)
 
 				elif isinstance(sectionnode, nodes.Element):
+
 					for toctreenode in traverse_in_section(sectionnode, addnodes.toctree):
 						item = toctreenode.copy()
 						entries.append(item)
@@ -215,17 +231,72 @@ class TocTreePlusCollector(TocTreeCollector):
 		app.env.toc_num_entries[docname] = numentries[0]
 
 
+def visit_desc(translator: LaTeXTranslator, node: addnodes.desc) -> None:
+	"""
+	Visit an :class:`addnodes.desc` node and add a custom table of contents label for the item, if required.
+
+	.. versionadded:: 0.3.0
+
+	:param translator:
+	:param node:
+	"""
+
+	translator.body.append("\n\n\\vspace{5px}\n\n\\begin{fulllineitems}\n")
+
+	# Add class, function and method directives to toctree.
+	if node.attributes["objtype"] in set(translator.config.toctree_plus_types):
+
+		attributes = node.children[0].attributes
+		if attributes["ids"]:
+			# Only want nodes with an anchor
+
+			title = texescape.escape(attributes.get("fullname", node.children[0].astext()))
+			sectionlevel = translator.sectionlevel + 1
+
+			if node.attributes["objtype"] in {"method", "attribute"}:
+				# TODO: remove special case
+				title = title.split('.', 1)[-1]
+				sectionlevel += 1
+
+			sectiontype = translator.sectionnames[sectionlevel]
+
+			translator.body.append(f"\\phantomsection\\stepcounter{{{sectiontype}}}\n")
+			translator.body.append(
+					"\\addcontentsline{toc}{%s}{\\protect\\numberline{\\the%s}{%s}}\n" %
+					(sectiontype, sectiontype, title)
+					)
+
+	if translator.table:
+		translator.table.has_problematic = True
+
+
+def depart_desc(translator: LaTeXTranslator, node: addnodes.desc) -> None:
+	"""
+	Visit an :class:`addnodes.desc` node.
+
+	.. versionadded:: 0.3.0
+
+	:param translator:
+	:param node:
+	"""
+
+	translator.body.append("\n\\end{fulllineitems}\n\n")
+
+
 def setup(app: Sphinx) -> Dict[str, Any]:
 	"""
 	Setup Sphinx Extension.
 
-	:param app:
+	:param app: The Sphinx application.
 	"""
 
 	# Set of types to add to toctree
-	app.add_config_value("toctree_plus_types", {"class", "function", "method"}, "env", [Set[str]])
+	app.add_config_value("toctree_plus_types", {"class", "function", "method"}, "env", [Iterable[str]])
 	app.add_env_collector(TocTreePlusCollector)
 
+	# Adds table of contents labels for LaTeX builder.
+	app.add_node(addnodes.desc, latex=(visit_desc, depart_desc), override=True)
+
 	return {
 			"version": __version__,
 			"parallel_read_safe": True,
